@@ -4,12 +4,13 @@ Fixes the huge problem of outdated field names in the api,
 that were leftover from during development
 """
 import re
+from datetime import datetime
 from math import ceil
 
 
 def _recognize_character_icon(url: str):
     """Recognizes a character's icon url and returns its name."""
-    exp = r'https://upload-os-bbs.mihoyo.com/game_record/genshin/character_(?:.*)_(\w+)(?:@2x|@3x)?.png'
+    exp = r'https://upload-os-bbs.mihoyo.com/game_record/genshin/character_(?:.*)_(\w+)(?:@\dx)?.png'
     match = re.fullmatch(exp,url)
     if match is None:
         return None
@@ -67,10 +68,12 @@ def prettify_spiral_abyss(data: dict):
         "icon":a["avatar_icon"],
         "id":a["avatar_id"],
     } for a in d]
+    todate = lambda x: datetime.fromtimestamp(int(x)).strftime("%Y-%m-%d")
+    totime = lambda x: datetime.fromtimestamp(int(x)).isoformat(' ')
     return {
         "season": data["schedule_id"],
-        "season_start_time": int(data["start_time"]),
-        "season_end_time": int(data["end_time"]),
+        "season_start_time": todate(data["start_time"]),
+        "season_end_time":   todate(data["end_time"]),
         "stats": {
             "total_battles": data["total_battle_times"],
             "total_wins": data["total_win_times"],
@@ -89,16 +92,16 @@ def prettify_spiral_abyss(data: dict):
             "floor": f["index"],
             "stars": f["star"],
             "max_stars": f["max_star"],
-            "start": int(f["levels"][0]["battles"][0]["timestamp"]),
+            "start": totime(f["levels"][0]["battles"][0]["timestamp"]),
             "icon": f["icon"],
-            "levels":[{
+            "chambers":[{
                 "chamber": l["index"],
                 "stars": l["star"],
                 "max_stars": l["max_star"],
                 "has_halves":len(l["battles"]) == 2,
                 "battles":[{
                     "half": b["index"],
-                    "timestamp": int(b["timestamp"]),
+                    "timestamp": totime(b["timestamp"]),
                     "characters":[{
                         "name": _recognize_character_icon(c["icon"]),
                         "rarity": c["rarity"],
@@ -121,6 +124,7 @@ def prettify_character(data: dict):
             "Traveler":"Aether" if "Boy" in data["icon"] else "Lumine",
             "Venti":"Barbatos",
             "Zhongli":"Morax",
+            "Fischl":"Amy",
             "Albedo":"Kreideprinz",
             "Tartaglia":"Childe",
         }.get(data["name"],None),
@@ -128,7 +132,7 @@ def prettify_character(data: dict):
         "element": data["element"] if data["name"]!="Traveler" else {
             71:"Anemo",
             91:"Geo"
-        }[data["constellations"][0]["id"]], # player elements
+        }[data["constellations"][0]["id"]], # traveler elements
         "level": data["level"],
         "ascension": ceil(data["level"]//10)-1,
         "friendship": data["fetter"],
@@ -148,15 +152,15 @@ def prettify_character(data: dict):
         },
         "artifacts": [{
             "name": a["name"],
-            "position_name": {
-                "Flower of Life":"flower",
-                "Plume of Death":"feather",
-                "Sands of Eon":"hourglass",
-                "Goblet of Eonothem":"goblet",
-                "Circlet of Logos":"crown",
-            }[a["pos_name"]],
-            "fancy_position_name": a["pos_name"],
-            "position_index": a["pos"],
+            "pos_name": {
+                1:"flower",
+                2:"feather",
+                3:"hourglass",
+                4:"goblet",
+                5:"crown",
+            }[a["pos"]],
+            "full_pos_name": a["pos_name"],
+            "pos": a["pos"],
             "rarity": a["rarity"],
             "level": a["level"],
             "set": {
@@ -174,7 +178,7 @@ def prettify_character(data: dict):
         } for a in data["reliquaries"]],
         "constellations": [{
             "name": c["name"],
-            "effect": c["effect"].replace('\\n','\n'),
+            "effect": c["effect"],
             "is_activated": c["is_actived"],
             "index": c["pos"],
             "icon": c["icon"],
@@ -192,28 +196,38 @@ def prettify_gacha_log(data: list):
         "name": i["name"],
         "rarity": int(i["rank_type"]),
         "time": i["time"],
+        "gacha_type": i["gacha_type"]
     } for i in data]
 
-def prettify_gacha_details(data: list):
+def prettify_gacha_items(data: list):
+    return [{
+        "name":i["name"],
+        "type":i["item_type"],
+        "rarity":i["rank_type"],
+        "id":10000000+int(i["item_id"])-1000 if len(i["item_id"])==4 else int(i["item_id"]),
+    } for i in data]
+
+def prettify_gacha_details(data: dict):
+    per = lambda p: None if p=='0%' else p[:-1].replace(',','.')
     fprobs = lambda l: [{
         "type": i["item_type"],
         "name": i["item_name"],
         "rarity": int(i["rank"]),
-        "is_up": i["is_up"],
-        "order_index": i["order_value"],
+        "is_up": bool(i["is_up"]),
+        "order_value": i["order_value"],
     } for i in l]
     fitems = lambda l: [{
         "type": i["item_type"],
         "name": i["item_name"],
         "element": {
-            "風":"Anemo",
+            "风":"Anemo",
             "火":"Pyro",
             "水":"Hydro",
             "雷":"Electro",
             "冰":"Cryo",
-            "山":"Geo",
+            "岩":"Geo",
             "？":"Dendro",
-            "":""
+            "":None
         }[i["item_attr"]],
         "icon": i["item_img"],
     } for i in l]
@@ -224,21 +238,26 @@ def prettify_gacha_details(data: list):
             "301":"Character Event Wish",
             "302":"Weapon Event Wish"
         }[data["gacha_type"]],
-        "banner": re.sub(r'<.*?>','',data["title"]).split('"')[1],
+        "gacha_type_id":int(data["gacha_type"]),
+        "banner": re.sub(r'<.*?>','',data["title"]).strip(),
         "title": data["title"],
         "content": data["content"],
-        "permanent": data["date_range"]=="Permanent",
-        "r5_up_prob": data["r5_up_prob"],
-        "r4_up_prob": data["r4_up_prob"],
-        "r5_prob": data["r5_prob"],
-        "r4_prob": data["r4_prob"],
-        "r3_prob": data["r3_prob"],
-        "r5_pity_prob": data["r5_baodi_prob"],
-        "r4_pity_prob": data["r4_baodi_prob"],
-        "r3_pity_prob": data["r3_baodi_prob"],
-        "r5_up_items": fitems(data["r5_up_items"]),
-        "r4_up_items": fitems(data["r4_up_items"]),
-        "r5_prob_list": fprobs(data["r5_prob_list"]),
-        "r4_prob_list": fprobs(data["r4_prob_list"]),
-        "r3_prob_list": fprobs(data["r3_prob_list"]),
+        "date_range":data["date_range"],
+        "permanent": data["gacha_type"]=="200",
+        "r5_up_prob": per(data["r5_up_prob"]), # probability for rate-up 5*
+        "r4_up_prob": per(data["r4_up_prob"]), # probability for rate-up 4*
+        "r5_prob": per(data["r5_prob"]), # probability for 5*
+        "r4_prob": per(data["r4_prob"]), # probability for 4*
+        "r3_prob": per(data["r3_prob"]), # probability for 3*
+        "r5_guarantee_prob": per(data["r5_baodi_prob"]), # probability for 5* incl. guarantee
+        "r4_guarantee_prob": per(data["r4_baodi_prob"]), # probability for 4* incl. guarantee
+        "r3_guarantee_prob": per(data["r3_baodi_prob"]), # probability for 3* incl. guarantee
+        "r5_up_items": fitems(data["r5_up_items"]), # list of 5* rate-up items that you can get from banner
+        "r4_up_items": fitems(data["r4_up_items"]), # list of 4* rate-up items that you can get from banner
+        "r5_items": fprobs(data["r5_prob_list"]), # list 5* of items that you can get from banner
+        "r4_items": fprobs(data["r4_prob_list"]), # list 4* of items that you can get from banner
+        "r3_items": fprobs(data["r3_prob_list"]), # list 3* of items that you can get from banner
+        "items": sorted(
+            fprobs(data["r5_prob_list"])+fprobs(data["r4_prob_list"])+fprobs(data["r3_prob_list"]),
+            key=lambda x: x["order_value"])
     }
